use tokio_postgres::Client as PgClient;
use std::sync::Arc;
use crate::models::administrator::Administrator;
use crate::storages::traits::AdministratorRepository;
use async_trait::async_trait;

pub struct PostgresAdministratorRepository {
    pub connection: Arc<PgClient>
}

#[async_trait]
impl AdministratorRepository for PostgresAdministratorRepository {
    async fn read(&self, id: String) -> Result<Administrator, String> {
        let result = self.connection.query(
            "SELECT * FROM administrator WHERE id = $1", &[&id.parse::<i32>().unwrap()]
        ).await;

        if let Ok(rows) = result {
            let row = rows.get(0).ok_or("Administrator not found".to_owned())?;
            Ok(Administrator {
                id: row.get::<&str, i32>("id").to_string(),
                username: row.get("username"),
                password: row.get("password"),
                name: row.get("name"),
                surname: row.get("surname"),
                middlename: row.get("middlename"),
                car_station_id: row.get::<&str, i32>("car_station_id").to_string()
            })
        } else {
            Err(result.unwrap_err().to_string())
        }
    }

    async fn find_by_username(&self, username: &str) -> Result<Administrator, String> {
        let result = self.connection.query("SELECT * FROM administrator WHERE username LIKE $1", &[&username]).await;
        
        if let Ok(rows) = result {
            let row = rows.get(0).ok_or("Administrator not found".to_owned())?;
            Ok(Administrator {
                id: row.get::<&str, i32>("id").to_string(),
                username: row.get("username"),
                password: row.get("password"),
                name: row.get("name"),
                surname: row.get("surname"),
                middlename: row.get("middlename"),
                car_station_id: row.get::<&str, i32>("car_station_id").to_string()
            })
        } else {
            Err(result.unwrap_err().to_string())
        }
    }
}